package Authen::WEIS::Consumer;

use 5.006;
use strict;
use warnings;

use Carp;
use CGI qw(escape);

use Authen::WEIS;
use Authen::WEIS::Util qw(:all);

use vars qw(@EXPORT @EXPORT_OK %EXPORT_TAGS @ISA $VERSION);
use fields qw(mode arg user pass);

require Exporter;
@ISA = qw(Exporter);
@EXPORT = ();
@EXPORT_OK = qw(build_weis_request);
%EXPORT_TAGS = (all => [ @EXPORT, @EXPORT_OK ]);

$VERSION = $Authen::WEIS::VERSION;


# returns consumer object
sub new (\$$) {
    my ($self, $args) = @_;
    unless (ref $self) {
	$self = fields::new($self);
    }
    foreach my $n (keys %$args) {
	my $v = $args->{$n};
	if ($n eq 'params') {
	    $self->{mode} = param($v, 'mode');
	    $self->{arg} = param($v, 'arg');
	    $self->{user} = param($v, 'user');
	    $self->{pass} = param($v, 'pass');
	} else {
	    my $ok = 0;
	    eval {
		$self->{$n} = $v;
		$ok = 1;
	    };
	    croak "unknown parameter: ``$n''" unless $ok;
	}
    }
    $self->{arg} = '' unless defined($self->{arg}); # set default
    return $self;
}

# accessors

sub mode (\$) {
    my $self = shift;
    return $self->is_valid && $self->{mode};
}

sub arg (\$) {
    my $self = shift;
    return $self->is_valid && $self->{arg};
}

sub user (\$) {
    my $self = shift;
    return $self->is_success && $self->{user};
}

sub pass (\$) {
    my $self = shift;
    return $self->is_success && $self->{pass};
}

# tests if contains a valid response
sub is_valid (\$) {
    my $self = shift;
    return unless defined($self->{mode});
    if ($self->{mode} eq 'ok' &&
	defined($self->{user}) &&
	defined($self->{pass})) {
	# ok
    } elsif ($self->{mode} eq 'cancel') {
	# ok
    } else {
	return;
    }
    return 1;
}

# returns if response is success or not
sub is_success (\$) {
    my $self = shift;
    return unless $self->is_valid;
    return unless ($self->mode eq 'ok');
    return 1;
}

# builds a request url
sub build_request ($) {
    my $args = $_[0];
    # check constraints
    my $self_url = $args->{self_url} or croak 'no self_url';
    croak('invalid self_url') unless is_valid_url($self_url);
    my $provider_url = $args->{provider_url} or croak 'no provider_url';
    croak('invalid provider_url') unless is_valid_url($provider_url);
    # build URL
    $provider_url =~ s/[\r\n]//g; # just in case
    my $url =
	sprintf('%s?ret=%s&arg=%s', $provider_url, escape($self_url),
		defined($args->{arg}) ? escape($args->{arg}) : '');
    
    return $url;
}

sub build_weis_request ($) {
    return build_request($_[0]);
}


1;
__END__

=head1 NAME

Authen::WEIS::Consumer - Perl implmentation of the WEIS protocol

=head1 SYNOPSIS

 use Authen::WEIS::Consumer qw(build_weis_request);

 # request crendentials
 redirect(
     -uri => build_weis_request({
         self_url => 'http://consumer.example.com/weis_consumer',
         provider_url => 'http://provider.example.org/weis_provider',
         arg => 'some_arbitrary_argument_if_any'
     });
 
 # extract the returned credentials
 my $consumer = Authen::WEIS::Consumer->new({ params => new CGI });
 if ($consumer->is_valid) {
     if ($consumer->is_success) {
	 $consumer->user; # get username
	 $consumer->pass; # get password
	 $consumer->arg;  # get arbitrary argument
     } else {
	 # provider refused to send username/password
     }
 } else {
     # invalid response
 }

=head1 DESCRIPTION

Authen::WEIS::Consumer is a implementation of the WEIS (Web Identity Syndication) protocol.

=head1 REQUESTING CREDENTIALS

To request credentials, redirect the user to a URL generated by calling I<build_weis_request>.

The function builds a URL used to request authentication credentials, taking the following parameters as a hash reference.

=over 4

=item provider_url

URL of the WEIS Provider (required)

=item self_url

URL to where the WEIS Provider should return the credentials (required)

=item arg

mandatory argument passed back to I<self_url> (optional)

=back

=head1 RECEIVING CREDENTIALS

Use the OO API to validate response and to extract the username / password.

=over 4

=item new({ args })

Constructor.  Takes the following parameters.

=over 8

=item params

a CGI object or a hash reference to query parameters (required)

=back

=item is_valid

Returns whether or not the response is valid.

=item is_success

Returns whether or not the response is a success.  May return false if the operation was cancelled by the user during his / her visit to the WEIS Provider page, or if the WEIS Provider refused to pass the credentials.

=item arg

Returns the mandatory argument passed through the WEIS provider, originated from I<build_weis_request>.

=item user

Returns the username passed from the WEIS Provider.

=item pass

Returns the password passed from the WEIS Provider.

=back

=head1 SEE ALSO

L<Authen::WEIS> L<Authen::WEIS::Provider>

=head1 AUTHOR

Kazuho Oku E<lt>kazuhooku ___at___ gmail.comE<gt>

Copyright (C) 2006  Cybozu Labs, Inc.

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself, either Perl version 5.8.7 or, at your option, any later version of Perl 5 you may have available.

=cut
