package Authen::WEIS::Provider;

use 5.006;
use strict;
use warnings;

use Carp;
use CGI qw(:html2 :form -no_xhtml escape escapeHTML);

use Authen::WEIS;
use Authen::WEIS::Util qw(:all);

use vars qw($VERSION);
use fields qw(ret arg);

$VERSION = $Authen::WEIS::VERSION;


# returns provider object
sub new (\$@) {
    my ($self, $args) = @_;
    unless (ref $self) {
	$self = fields::new($self);
    }
    if (defined $args) {
	foreach my $n (keys %$args) {
	    my $v = $args->{$n};
	    if ($n eq 'params') {
		$self->{ret} = param($v, 'ret');
		$self->{arg} = param($v, 'arg');
	    } else {
		my $ok = 0;
		eval {
		    $self->{$n} = $v;
		    $ok = 1;
		};
		croak "unknown parameter: ``$n''" unless $ok;
	    }
	}
    }
    $self->{ret} = undef unless defined($self->{ret});
    $self->{arg} = '' unless defined($self->{arg});
    return $self;
}

# accessors
sub ret (\$) {
    my $self = shift;
    return $self->is_valid && $self->{ret};
}

sub arg (\$) {
    my $self = shift;
    return $self->is_valid && $self->{arg};
}

# tests if contains a valid request
sub is_valid (\$) {
    my $self = shift;
    return unless is_valid_url($self->{ret});
    return 1;
}

# builds response
sub build_response (\$$) {
    my ($self, $args) = @_;
    # check precondition
    croak 'no user' unless defined($args->{user});
    croak 'no password' unless defined($args->{pass});
    # check constraints
    return unless $self->is_valid;
    # build HTML
    my $charset = $args->{encoding} || 'us-ascii';
    my $html =
	start_html(-title => $args->{title} || 'WEIS Response',
		   -head => meta({-http_equiv => 'content-type',
				  -content    => "text/html; $charset"}));
    $html .=
	start_form(-method => 'POST', -action => $self->{ret}) .
	_hidden('mode', 'ok') .
	_hidden('arg', $self->{arg}) .
	_hidden('user', $args->{user}) .
	_hidden('pass', $args->{pass}) .
	'<noscript><input type="submit" value="Link the Websites"></noscript>' .
	end_form;
    $html .=
	'<script>document.forms[0].submit()</script>';
    $html .=
	end_html;
    return $html;
}

# returns cancel url
sub build_cancel_url (\$) {
    my $self = shift;
    # check constraints
    return unless $self->is_valid;
    # build url
    my $url = $self->{ret};
    $url =~ s/[\r\n]//g; # just in case
    $url .= '?mode=cancel&arg=' . escape($self->{arg});
    return $url;
}

# builds a hidden input field
sub _hidden ($$) {
    my ($name, $value) = @_;
    return
	sprintf('<input type="hidden" name="%s" value="%s">',
		escapeHTML($name), escapeHTML($value));
}
	
1;
__END__

=head1 NAME

Authen::WEIS::Provider - Perl implementation of the WEIS protocol

=head1 SYNOPSIS

 use Authen::WEIS::Provider;

 my $provider = Authen::WEIS::Provider->new({ params => new CGI });
 
 if ($provider->is_valid) {
     # check cookies, etc. to determine the user, and send back a
     # username / password pair for accessing the resource
     print
	 header(-pragma => 'no-cache') .
	 $provider->build_response({
	     user => $user,
	     password => $password });
 }

=head1 DESCRIPTION

Authen::WEIS::Provider is a implementation of the WEIS (Web Identity Syndication) protocol.

The package provides an OO-style API to handle credential tranfer requests.

=over 4

=item new({ args })

Constructor.  Takes the following arguments.

=over 8

=item params

a CGI object or hash reference to query parameters (required)

=back

=item is_valid

Returns whether or not the request is valid.

=item build_response({ args })

Builds a response that will send back the authentication credentials (username and password) to the WEIS Consumer.  The subroutine takes the following arguments.

=over 8

=item user

username (required)

=item pass

password (required)

=back

=head1 SEE ALSO

L<Authen::WEIS> L<Authen::WEIS::Provider>

=head1 AUTHOR

Kazuho Oku E<lt>kazuhooku ___at___ gmail.comE<gt>

Copyright (C) 2006 Cybozu Labs, Inc.

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself, either Perl version 5.8.7 or, at your option, any later version of Perl 5 you may have available.

=cut
